from django.test.testcases import TestCase
from django.contrib.auth.models import Group, User
from djinn_auth.models import Role
from djinn_auth.utils import set_local_role, get_local_roles, has_local_role, \
    assign_local_role, has_user_local_role


class LocalRoleTest(TestCase):

    def setUp(self):

        self.content = Group.objects.create(name="Foo")
        self.owner = Role.objects.create(name="owner")

    def test_set_local_role(self):

        tjibbe = User.objects.create(username="Tjibbe")
        gurbe = User.objects.create(username="Gurbe")

        self.assertFalse(has_local_role(tjibbe, self.content, self.owner))
        self.assertFalse(has_local_role(gurbe, self.content, self.owner))

        set_local_role(tjibbe, self.content, self.owner)

        self.assertTrue(has_local_role(tjibbe, self.content, self.owner))
        self.assertFalse(has_local_role(gurbe, self.content, self.owner))

        set_local_role(gurbe, self.content, self.owner)

        self.assertFalse(has_local_role(tjibbe, self.content, self.owner))
        self.assertTrue(has_local_role(gurbe, self.content, self.owner))

    def test_set_local_role_to_group(self):

        tjibbe = User.objects.create(username="Tjibbe")
        tjibbes = Group.objects.create(name="Tjibbes")

        self.assertFalse(has_user_local_role(tjibbe, self.content, self.owner))

        set_local_role(tjibbes, self.content, self.owner)

        self.assertFalse(has_user_local_role(tjibbe, self.content, self.owner))

        tjibbe.groups.add(tjibbes)

        self.assertTrue(has_user_local_role(tjibbe, self.content, self.owner))

    def test_assign_local_role(self):

        tjibbe = User.objects.create(username="Tjibbe")
        gurbe = User.objects.create(username="Gurbe")

        self.assertFalse(has_local_role(tjibbe, self.content, self.owner))
        self.assertFalse(has_local_role(gurbe, self.content, self.owner))

        assign_local_role(tjibbe, self.content, self.owner)

        self.assertTrue(has_local_role(tjibbe, self.content, self.owner))
        self.assertFalse(has_local_role(gurbe, self.content, self.owner))

        assign_local_role(gurbe, self.content, self.owner)

        self.assertTrue(has_local_role(tjibbe, self.content, self.owner))
        self.assertTrue(has_local_role(gurbe, self.content, self.owner))

    def test_get_local_roles(self):

        tjibbe = User.objects.create(username="Tjibbe")
        gurbe = User.objects.create(username="Gurbe")

        self.assertEquals(0, len(get_local_roles(self.content)))
        self.assertEquals(0, len(get_local_roles(self.content, self.owner)))

        set_local_role(tjibbe, self.content, self.owner)
        assign_local_role(gurbe, self.content, self.owner)

        self.assertEquals(2, len(get_local_roles(self.content)))
        self.assertEquals(2, len(get_local_roles(self.content, self.owner)))



from django.contrib.auth.models import Permission
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Role(models.Model):

    """ A role is really nothing but a bundle of permissions """

    name = models.CharField(_('name'), max_length=80, unique=True)
    permissions = models.ManyToManyField(Permission,
                                         verbose_name=_('permissions'))

    class Meta:

        app_label = "djinn_auth"
        ordering = ['name']

    def __unicode__(self):

        return self.name

    def add_permission(self, permission):

        """ Add the permission if it's not already there """

        if not self.permissions.filter(codename=permission.codename).exists():
            self.permissions.add(permission)



from django.db import models
from django.contrib.auth.models import Permission
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType


class LocalPermission(models.Model):

    """Local permission for given model instance. Can be assigned either
    to user or to usergroup.

    """

    instance_ct = models.ForeignKey(ContentType, related_name='+')
    instance_id = models.PositiveIntegerField()
    instance = generic.GenericForeignKey('instance_ct', 'instance_id')

    assignee_ct = models.ForeignKey(ContentType, related_name='+')
    assignee_id = models.PositiveIntegerField()
    assignee = generic.GenericForeignKey('assignee_ct', 'assignee_id')

    permission = models.ForeignKey(Permission)

    class Meta:

        app_label = "djinn_auth"

    def __unicode__(self):

        return u"%s has %s for %s" % (self.assignee, self.permission,
                                      self.content)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'django',
    ]

setup(name='djinn_auth',
      version="1.0.0",
      description='Djinn Intranet auth module, role based',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
          "Development Status :: 3 - Alpha",
          "Framework :: Django",
          "Intended Audience :: Developers",
          "License :: Freely Distributable",
          "Programming Language :: Python",
          "Topic :: Internet :: WWW/HTTP :: Site Management",
          "Topic :: Software Development :: Libraries :: "
          "Application Frameworks"
      ],
      author='PythonUnited',
      author_email='info@pythonunited.com',
      license='beer-ware',
      url='https://github.com/PythonUnited/djinn_auth',
      keywords='Djinn role base auth',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      test_suite="djinn_auth",
      entry_points="""\
      [djinn.app]
      urls=djinn_auth:get_urls
      """
      )
